pub enum Error {
Syntax(Box<Error<Rule>>),
Unsupported(&'static str),
Overflow {
value: i16,
expected_bounds: RangeInclusive<i16>,
},
InvalidExtendedTime {
hour: u8,
minutes: u8,
},
InvertedYearRange {
start: Year,
end: Year,
step: u16,
},
InvertedWeekRange {
start: WeekNum,
end: WeekNum,
step: u8,
},
GrammarUnexpectedToken {
rule: Rule,
unexpected: Rule,
},
GrammarLogic {
rule: Rule,
invariant: &'static str,
},
}Variants
Syntax(Box<Error<Rule>>)
Could not parse the expression. It is not conformant with the grammar defined in the wiki: https://wiki.openstreetmap.org/wiki/Key:opening_hours/specification
Unsupported(&'static str)
Use of an unsupported feature.
Overflow
Some number has overflowed.
InvalidExtendedTime
Extended time has overflowed.
InvertedYearRange
A year range that starts after it ends.
InvertedWeekRange
A week range that starts after it ends.
GrammarUnexpectedToken
This should never be built at runtime if the grammar implementation is sound.
GrammarLogic
This should never be built at runtime if the grammar implementation is sound.
Implementations
Sourceimpl Error
impl Error
Sourcepub fn is_implementation_error(&self) -> bool
pub fn is_implementation_error(&self) -> bool
If this is true, this is an error that should not be raised as long as the implementation is sound. If this kind of error occurs, you can report it here : https://github.com/remi-dupre/opening-hours-rs/issues