pub fn parse(data: &str) -> Result<OpeningHoursExpression>Expand description
Parse the expression with a default parser configuration (no warning handling).
pub fn parse(data: &str) -> Result<OpeningHoursExpression>Parse the expression with a default parser configuration (no warning handling).