pub struct YearRange { /* private fields */ }Expand description
A year range that ensures that bounds are always in the right order
Implementations
Sourceimpl YearRange
impl YearRange
Sourcepub fn new(range: RangeInclusive<Year>, step: u16) -> Option<Self>
pub fn new(range: RangeInclusive<Year>, step: u16) -> Option<Self>
Create a new YearRange. Return None if the bounds are in the wrong order.
Sourcepub fn into_parts(&self) -> (RangeInclusive<Year>, u16)
pub fn into_parts(&self) -> (RangeInclusive<Year>, u16)
Extract range and step from this object.